@@ -701,23 +701,23 @@ Complex chebyInnerProduct(const BasisFunc& f, const BasisFunc& g,
 
 // ==================================================================
 // switchable norms
-Real norm(const BasisFunc& phi, normType n, bool normalize) {
+Real norm(const BasisFunc& phi, NormType n, bool normalize) {
   return (n==L2NORM) ? L2Norm(phi,normalize) : chebyNorm(phi,normalize);
 }
 
-Real norm2(const BasisFunc& phi, normType n, bool normalize) {
+Real norm2(const BasisFunc& phi, NormType n, bool normalize) {
   return (n==L2NORM) ? L2Norm2(phi,normalize) : chebyNorm2(phi,normalize);
 }
 
-Real dist2(const BasisFunc& f, const BasisFunc& g, normType n, bool nrmlz) {
+Real dist2(const BasisFunc& f, const BasisFunc& g, NormType n, bool nrmlz) {
   return (n==L2NORM) ? L2Dist2(f,g,nrmlz) : chebyDist2(f,g,nrmlz);
 }
 
-Real dist(const BasisFunc& f, const BasisFunc& g,  normType n, bool nrmlz) {
+Real dist(const BasisFunc& f, const BasisFunc& g,  NormType n, bool nrmlz) {
   return (n==L2NORM) ? L2Dist(f,g,nrmlz) : chebyDist(f,g,nrmlz);
 }
 
-Complex innerProduct(const BasisFunc& f, const BasisFunc& g, normType n,
+Complex innerProduct(const BasisFunc& f, const BasisFunc& g, NormType n,
 		       bool nrmlz) {
   return (n==L2NORM) ? L2InnerProduct(f,g,nrmlz) : chebyInnerProduct(f,g,nrmlz);
 }
