@@ -24,7 +24,8 @@
  */
 
 // Wordexp is used in fftw_savewisdom and load, to expand possible home
-// dir symbol ~. COmment it out if if auses you trouble.
+// dir symbol ~. Comment it out if it causes you trouble.
+
 #include <wordexp.h>
 
 #include <fstream>
@@ -33,6 +34,12 @@
 
 const Real QUADRATURE_EPSILON=1e-17;
 
+// Integral_(-1)^1 Tm(y) Tn(y) dy
+Real chebyIP(int m, int n) { 
+  return ((m+n) % 2 == 1) ? 0 :
+    (1.0-m*m-n*n)/((1.0+m-n)*(1.0-m+n)*(1.0+m+n)*(1.0-m-n));
+}
+
 Real legendre(int n, Real x) {
   Real p=1.0;
   Real q=0.0;
@@ -251,22 +258,24 @@ void ChebyCoeff::makePhysical() {
   t.makePhysical(*this);
 }
 void ChebyCoeff::makeState(fieldstate s){ 
-  if (s!=state_) {
+  if (state_ != s) {       // need to change state?
     ChebyTransform t(N_);
-    if (s==Physical)
+    if (state_==Physical)  // state is Physical; change to Spectral
       t.chebyfft(*this);
     else
-      t.ichebyfft(*this);
+      t.ichebyfft(*this);  // state is Spectral; change to Physical
   }
 }
 
 
 void ChebyCoeff::randomize(Real decay, bool a_dirichlet, bool b_dirichlet) {
-  Real mag=1.0;
   fieldstate startState = state_;
   state_ = Spectral;
-  for (int i=0; i<N_; ++i) 
-    data_[i] = (mag *= decay) * randomReal();
+  Real pow_decay_n = 1.0;
+  for (int n=0; n<N_; ++n) {
+    data_[n] = pow_decay_n * randomReal();
+    pow_decay_n *= decay;
+  }
   
   if (a_dirichlet && b_dirichlet) {
     data_[1] -= 0.5*(eval_b() - eval_a());
@@ -742,24 +751,24 @@ Real chebyInnerProduct(const ChebyCoeff& u, const ChebyCoeff& v, bool normalize)
 
 // ===========================================================
 // switchable norms
-Real norm2(const ChebyCoeff& u, normType n, bool normalize) {
+Real norm2(const ChebyCoeff& u, NormType n, bool normalize) {
   return (n==L2NORM) ? L2Norm2(u,normalize) : chebyNorm2(u,normalize);
 }
 ; 
-Real dist2(const ChebyCoeff& u, const ChebyCoeff& v, normType n,
+Real dist2(const ChebyCoeff& u, const ChebyCoeff& v, NormType n,
 	   bool normalize) {
   return (n==L2NORM) ? L2Dist2(u,v,normalize) : chebyDist2(u,v,normalize);
 }
 
-Real norm(const ChebyCoeff& u, normType n, bool normalize) {
+Real norm(const ChebyCoeff& u, NormType n, bool normalize) {
   return (n==L2NORM) ? L2Norm(u,normalize) : chebyNorm(u,normalize);
 }
-Real dist(const ChebyCoeff& u, const ChebyCoeff& v, normType n, 
+Real dist(const ChebyCoeff& u, const ChebyCoeff& v, NormType n, 
 	  bool normalize) {
   return (n==L2NORM) ? L2Dist(u,v,normalize) : chebyDist(u,v,normalize);
 }
 Real innerProduct(const ChebyCoeff& u, const ChebyCoeff& v, 
-		  normType n,  bool normalize) {
+		  NormType n,  bool normalize) {
   return (n==L2NORM) ? L2InnerProduct(u,v,normalize):chebyInnerProduct(u,v,normalize);
 }
 
@@ -859,12 +868,13 @@ void ComplexChebyCoeff::makePhysical() {
   t.makePhysical(*this);
 }
 void ComplexChebyCoeff::makeState(fieldstate s){ 
-  if (s!=re.state()) {
+  assert(re.state() == im.state());
+  if (re.state() != s) {       // need to change state
     ChebyTransform t(re.numModes());
-    if (s==Physical)
+    if (re.state()==Physical)  // state is Physical, change to Spectral
       t.chebyfft(*this);
     else
-      t.ichebyfft(*this);
+      t.ichebyfft(*this);      // state is Spectral, change to Physical
   }
 }
 
@@ -1379,25 +1389,25 @@ Complex chebyInnerProduct(const ComplexChebyCoeff& u,
 
 // ===========================================================
 // switchable norms
-Real norm2(const ComplexChebyCoeff& u, normType n, bool normalize) {
+Real norm2(const ComplexChebyCoeff& u, NormType n, bool normalize) {
   return (n == L2NORM) ? L2Norm2(u,normalize) : chebyNorm2(u,normalize);
 }
 
 Real dist2(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
-	   normType n, bool normalize) {
+	   NormType n, bool normalize) {
   return (n == L2NORM) ? L2Dist2(u,v,normalize) : chebyDist2(u,v,normalize);
 }
 
-Real norm(const ComplexChebyCoeff& u, normType n, bool normalize) {
+Real norm(const ComplexChebyCoeff& u, NormType n, bool normalize) {
   return (n == L2NORM) ? L2Norm(u,normalize) : chebyNorm(u,normalize);
 }
 
 Real dist(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
-	  normType n, bool normalize) {
+	  NormType n, bool normalize) {
   return (n == L2NORM) ? L2Dist(u,v,normalize) : chebyDist(u,v,normalize);
 }
 Complex innerProduct(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
-		  normType n,  bool normalize) {
+		  NormType n,  bool normalize) {
   return (n == L2NORM) ? L2InnerProduct(u,v,normalize) : chebyInnerProduct(u,v,normalize);
 }
 
