@@ -43,6 +43,7 @@ FlowField::FlowField()
   Lz_(0),
   a_(0),
   b_(0),
+  dealiasIO_(false),
   rdata_(0),
   cdata_(0),
   q_(0),
@@ -72,6 +73,7 @@ FlowField::FlowField(int Nx, int Ny, int Nz, int Nd, Real Lx, Real Lz,
   Lz_(0),
   a_(0),
   b_(0),
+  dealiasIO_(false),
   rdata_(0),
   cdata_(0),
   q_(0),
@@ -103,6 +105,7 @@ FlowField::FlowField(const FlowField& f)
   Lz_(0),
   a_(0),
   b_(0),
+  dealiasIO_(f.dealiasIO_),
   rdata_(0),
   cdata_(0),
   q_(0),
@@ -120,6 +123,7 @@ FlowField::FlowField(const FlowField& f)
 {
   resize(f.Nx_, f.Ny_, f.Nz_, f.Nd_, f.Lx_, f.Lz_, f.a_, f.b_);
   setState(f.xzstate_, f.ystate_);
+  dealiasIO_ = f.dealiasIO_;
   int N = Nd_*Ny_*Nx_*Nzpad_;
   for (int i=0; i<N; ++i)
     rdata_[i] = f.rdata_[i];
@@ -139,6 +143,7 @@ FlowField::FlowField(const string& filebase)
   Lz_(0.0),
   a_(0.0),
   b_(0.0),
+  dealiasIO_(false),
   rdata_(0),
   cdata_(0),
   q_(0),
@@ -172,17 +177,49 @@ FlowField::FlowField(const string& filebase)
   read(is, Lz_);
   read(is, a_);
   read(is, b_);
-
+  read(is, dealiasIO_);
   resize(Nx_, Ny_, Nz_, Nd_, Lx_, Lz_, a_, b_);
 
-  int N = Nd_*Ny_*Nx_*Nzpad_;
-  for (int i=0; i<N; ++i)
-    read(is, rdata_[i]);
+  // Read data only for non-aliased modes, assume 0 for aliased.
+  if (dealiasIO_ == true && xzstate_ == Spectral) {
+    int Nxd=2*(Nx_/6);
+    int Nzd=2*(Nz_/3)+1;
+
+    // In innermost loop, array index is (nz + Nzpad2_*(nx + Nx_*(ny + Ny_*i))),
+    // which is the same as the FlowField::flatten function.
+    for (int i=0; i<Nd_; ++i) {
+      for (int ny=0; ny<Ny_; ++ny) {
+
+	for (int nx=0; nx<=Nxd; ++nx) {
+	  for (int nz=0; nz<=Nzd; ++nz)
+	    read(is, rdata_[flatten(nx,ny,nz,i)]);
+	  for (int nz=Nzd+1; nz<Nzpad_; ++nz)
+	    rdata_[flatten(nx,ny,nz,i)] = 0.0;
+	}
+	for (int nx=Nxd+1; nx<=Nxd; ++nx) 
+	  for (int nz=0; nz<=Nzpad_; ++nz)
+	    rdata_[flatten(nx,ny,nz,i)] = 0.0;
+	
+	for (int nx=Nx_-Nxd; nx<Nx_; ++nx) {
+	  for (int nz=0; nz<=Nzd; ++nz)
+	    read(is, rdata_[flatten(nx,ny,nz,i)]);
+	  for (int nz=Nzd+1; nz<Nzpad_; ++nz)
+	    rdata_[flatten(nx,ny,nz,i)] = 0.0;
+	}
+      }
+    }
+  }
+  else {
+    int N = Nd_*Ny_*Nx_*Nzpad_;
+    for (int i=0; i<N; ++i)
+      read(is, rdata_[i]);
+  }
 }
 
 FlowField& FlowField::operator=(const FlowField& f) {
   resize(f.Nx_, f.Ny_, f.Nz_, f.Nd_, f.Lx_, f.Lz_, f.a_, f.b_);
   setState(f.xzstate_, f.ystate_);
+  dealiasIO_ = f.dealiasIO_;
   int Ntotal = Nx_ * Ny_ * Nzpad_ * Nd_;
   for (int i=0; i<Ntotal; ++i)
     rdata_[i] = f.rdata_[i];
@@ -296,32 +333,50 @@ void FlowField::interpolate(const FlowField& u) {
   assert(Nd_ == u.Nd_);
   u.assertState(Spectral, Spectral);
 
-  setState(Spectral, Physical);
-  setToZero();
-  
-  ComplexChebyCoeff uprof(u.Ny(), u.a(), u.b(), Spectral);
-  ComplexChebyCoeff tprof(Ny_, a_, b_, Physical);
-
   int kxmn = Greater(kxmin(), u.kxmin());
   int kxmx = lesser(kxmax(), u.kxmax());
   int kzmn = Greater(kzmin(), u.kzmin()); // == 0
   int kzmx = lesser(kzmax(), u.kzmax());
   int uNy = u.Ny();
-  int tNy = Ny_;
 
-  for (int i=0; i<Nd_; ++i) {
-    // Copy kx==0 coeffs.
-    for (int kx=kxmn; kx<=kxmx; ++kx) {
-      int unx = u.nx(kx);
-      int tnx = nx(kx);
-      for (int kz=kzmn; kz<=kzmx; ++kz) {
-	int unz = u.nz(kz);
-	int tnz = nz(kz);
-	for (int uny=0; uny<uNy; ++uny) 
-	  uprof.set(uny, u.cmplx(unx, uny, unz, i));
-	tprof.interpolate(uprof);
-	for (int tny=0; tny<tNy; ++tny) 
-	  cmplx(tnx, tny, tnz, i) = tprof[tny];
+  setState(Spectral, Spectral);
+  setToZero();
+
+  // chebyshev discretization is the same, copy coeffs
+  if (Ny_ == uNy &&  a_ == u.a_ && b_ == u.b_) {
+    for (int i=0; i<Nd_; ++i) {
+      for (int kx=kxmn; kx<=kxmx; ++kx) {
+	int unx = u.nx(kx);
+	int tnx = nx(kx);
+	for (int kz=kzmn; kz<=kzmx; ++kz) {
+	  int unz = u.nz(kz);
+	  int tnz = nz(kz); 
+	  for (int ny=0; ny<uNy; ++ny) 
+	    cmplx(tnx, ny, tnz, i) = u.cmplx(unx, ny, unz, i);
+	}
+      }
+    }
+  }  
+  else {
+    // chebyshev discretization is the different, interpolate coeffs
+    ComplexChebyCoeff uprof(u.Ny(), u.a(), u.b(), Spectral);
+    ComplexChebyCoeff tprof(Ny_, a_, b_, Physical);
+    ChebyTransform trans(Ny_);
+    
+    for (int i=0; i<Nd_; ++i) {
+      for (int kx=kxmn; kx<=kxmx; ++kx) {
+	int unx = u.nx(kx);
+	int tnx = nx(kx);
+	for (int kz=kzmn; kz<=kzmx; ++kz) {
+	  int unz = u.nz(kz);
+	  int tnz = nz(kz); 
+	  for (int uny=0; uny<uNy; ++uny) 
+	    uprof.set(uny, u.cmplx(unx, uny, unz, i));
+	  tprof.interpolate(uprof);
+	  tprof.makeSpectral(trans);
+	  for (int tny=0; tny<Ny_; ++tny) 
+	    cmplx(tnx, tny, tnz, i) = tprof[tny];
+	}
       }
     }
   }
@@ -345,7 +400,6 @@ void FlowField::reflect(const FlowField& u) {
   int kzmn = Greater(kzmin(), u.kzmin()); // == 0
   int kzmx = lesser(kzmax(), u.kzmax());
   int uNy = u.Ny();
-  int tNy = Ny_;
 
   for (int i=0; i<Nd_; ++i) {
     // Copy kx==0 coeffs.
@@ -359,7 +413,7 @@ void FlowField::reflect(const FlowField& u) {
 	for (int uny=0; uny<uNy; ++uny) 
 	  uprof.set(uny, u.cmplx(unx, uny, unz, i));
 	tprof.reflect(uprof,p);
-	for (int tny=0; tny<tNy; ++tny) 
+	for (int tny=0; tny<Ny_; ++tny) 
 	  cmplx(tnx, tny, tnz, i) = tprof[tny];
       }
     }
@@ -738,11 +792,14 @@ void FlowField::addPerturbation(int kx, int kz, Real mag, Real decay) {
   ComplexChebyCoeff v(Ny_, a_, b_, Spectral);
   ComplexChebyCoeff w(Ny_, a_, b_, Spectral);
   randomProfile(u,v,w, kx, kz, Lx_, Lz_, mag, decay);
+  int nx_ = nx(kx);
+  int nz_ = nx(kz);
   for (int ny=0; ny<Ny_; ++ny) {
-    cmplx(kx, ny, kz, 0) += u[ny];
-    cmplx(kx, ny, kz, 1) += v[ny];
-    cmplx(kx, ny, kz, 2) += w[ny];
+    cmplx(nx_, ny, nz_, 0) += u[ny];
+    cmplx(nx_, ny, nz_, 1) += v[ny];
+    cmplx(nx_, ny, nz_, 2) += w[ny];
   }
+  /******************************************
   // Add complex conjugate to negative kx counterpart to keep field real.
   if (kx > 0) {
     randomProfile(u,v,w, -kx, kz, Lx_, Lz_, mag, decay);
@@ -752,24 +809,50 @@ void FlowField::addPerturbation(int kx, int kz, Real mag, Real decay) {
       cmplx(Nx_-kx, ny, kz, 2) += w[ny];
     }
   }
+  *******************************************/
   return;
 }
 
-void FlowField::addPerturbations(int kxmax, int kzmax, Real mag, Real decay) {
+void FlowField::addPerturbations(int kxmx, int kzmx, Real mag, Real decay) {
   assertState(Spectral, Spectral);
   if (mag == 0.0)
     return;
 
-  // Add a dive-free perturbation to the base flow.
-  for (int kx=0; kx<=kxmax; ++kx) 
-    for (int kz=0; kz<=kzmax; ++kz) {
-      Real norm = pow(10.0, abs(2*pi*kx/Lx_) + abs(2*pi*kz/Lz_));
+  int kxmn = Greater(-kxmx, kxmin());
+  kxmx = lesser(kxmx, kxmax());
+  kzmx = lesser(kzmx, kzmax());
+  // Add a div-free perturbation to the base flow.
+  for (int kx=kxmn; kx<=kxmx; ++kx) 
+    for (int kz=0; kz<=kzmx; ++kz) {
+      //Real norm = pow(10.0, -(abs(2*pi*kx/Lx_) + abs(2*pi*kz/Lz_)));
+      Real norm = pow(decay, 2*(abs(kx) + abs(kz)));
       if (!(kx==0 && kz==0))
-	addPerturbation(kx,kz,mag/norm, decay);
+	addPerturbation(kx, kz, mag*norm, decay);
     }
+  makePhysical();
+  makeSpectral();
   return;
 }
 
+void FlowField::addPerturbations(Real mag, Real decay) {
+  assertState(Spectral, Spectral);
+  if (mag == 0.0)
+    return;
+
+  // Add a div-free perturbation to the base flow.
+  for (int nx=0; nx<numXmodes(); ++nx) {
+    int kx_ = kx(nx);
+    for (int nz=0; nz<numZmodes(); ++nz) {
+      int kz_ = kz(nz);
+      Real norm = pow(decay, 2*(abs(kx_) + abs(kz_)));
+      addPerturbation(kx_, kz_, mag*norm, decay);
+    }
+  }
+  makePhysical();
+  makeSpectral();
+  return;
+}
+void FlowField::dealiasIO(bool b) {dealiasIO_ = b;}
 
 void FlowField::setToZero() {
   int Ntotal = Nx_*Ny_*Nzpad_*Nd_;
@@ -909,7 +992,7 @@ void FlowField::saveSpectrum(const string& filebase, int i, int ny) const {
   for (int nx=0; nx<numXmodes(); ++nx) {
     for (int nz=0; nz<numZmodes(); ++nz) {
       if (sum) 
-	os << energy(kx(nx),kz(nz)) << ' ';
+	os << sqrt(energy(kx(nx),kz(nz))) << ' ';
       else {
 	Complex f = this->cmplx(nx, ny, nz, i);
 	os << Re(f) << ' ' << Im(f) << ' ';
@@ -927,6 +1010,19 @@ void FlowField::saveSpectrum(const string& filebase) const {
 
   assert(xzstate_ == Spectral && ystate_ == Spectral);
   BasisFunc prof(Ny_, 0, 0, Lx_, Lz_, a_, b_, Spectral);
+ 
+  int Nx = numXmodes();
+  int Nz = numZmodes();
+  
+  for (int nx=0; nx<Nx; ++nx) {
+    int kx_=kx(nx);
+    for (int nz=0; nz<Nz; ++nz) {
+      int kz_=kz(nz);
+      os << energy(kx_,kz_) << ' ';
+    }
+    os << endl;
+  }
+  /*****************
   int kxmn = kxmin();
   int kxmx = kxmax();
   int kzmn = kzmin();
@@ -937,6 +1033,7 @@ void FlowField::saveSpectrum(const string& filebase) const {
     }
     os << endl;
   }
+  *****************/
   os.close();
 }
 
@@ -1012,11 +1109,34 @@ void FlowField::binarySave(const string& filebase) const {
   write(os, Lz_);
   write(os, a_);
   write(os, b_);
-  
-  int Ntotal = Nd_* Nx_ * Ny_ * Nzpad_;
-  for (int i=0; i<Ntotal; ++i)
-    write(os, rdata_[i]);
+  write(os, dealiasIO_);
 
+  // Write data only for non-aliased modes.
+  if (dealiasIO_ && xzstate_ == Spectral) {
+    int Nxd=2*(Nx_/6);
+    int Nzd=2*(Nz_/3)+1;
+
+    // In innermost loop, array index is (nz + Nzpad2_*(nx + Nx_*(ny + Ny_*i))),
+    // which is the same as the FlowField::flatten function.
+    for (int i=0; i<Nd_; ++i) {
+      for (int ny=0; ny<Ny_; ++ny) {
+
+	for (int nx=0; nx<=Nxd; ++nx) {
+	  for (int nz=0; nz<=Nzd; ++nz)
+	    write(os, rdata_[flatten(nx,ny,nz,i)]);
+	}
+	for (int nx=Nx_-Nxd; nx<Nx_; ++nx) {
+	  for (int nz=0; nz<=Nzd; ++nz)
+	    write(os, rdata_[flatten(nx,ny,nz,i)]);
+	}
+      }
+    }
+  }
+  else {
+    int Ntotal = Nd_* Nx_ * Ny_ * Nzpad_;
+    for (int i=0; i<Ntotal; ++i)
+      write(os, rdata_[i]);
+  }
 }
 
 void FlowField::dump() const {
@@ -1248,6 +1368,63 @@ Real L2Dist2(const FlowField& u, const FlowField& v, bool normalize) {
   return sum;
 }
 
+Real L2Norm2(const FlowField& u, int kxmax, int kzmax, bool normalize) {
+  alwaysAssert(u.ystate()==Spectral);
+  alwaysAssert(u.xzstate()==Spectral);
+  Real sum = 0.0;
+  
+  // EFFICIENCY: would be better with smarter looping, but prob not worth trouble.
+  ComplexChebyCoeff prof(u.Ny(), u.a(), u.b(), Spectral);
+  for (int i=0; i<u.vectorDim(); ++i)
+    for (int nx=0; nx<u.numXmodes(); ++nx) {
+      if (abs(u.kx(nx)) > kxmax)
+	continue;
+      Real cz = 1.0; // cz = 2 for kz>0 to take account of kz<0 ghost modes
+      for (int nz=0; nz<u.numZmodes(); ++nz) {
+	if (abs(u.kz(nz)) > kzmax)
+	  continue;
+	for (int ny=0; ny<u.numYmodes(); ++ny) 
+	  prof.set(ny, u.cmplx(nx,ny,nz,i));
+	sum += cz*L2Norm2(prof, normalize);
+	cz = 2.0;
+      }
+    }
+  if (!normalize)
+    sum *= u.Lx()*u.Lz();
+  return sum;
+}
+
+Real L2Norm(const FlowField& u, int kxmax, int kzmax, bool normalize) {
+  return sqrt(L2Norm2(u, kxmax, kzmax, normalize));
+}
+Real L2Dist(const FlowField& u, const FlowField& v, int kxmax, int kzmax, bool normalize) {
+  return sqrt(L2Dist2(u, v, kxmax, kzmax, normalize));
+}
+
+Real L2Dist2(const FlowField& u, const FlowField& v, int kxmax, int kzmax, bool normalize) {
+  Real sum = 0.0;
+  alwaysAssert(u.congruent(v));
+  alwaysAssert(u.xzstate()==Spectral && u.ystate()==Spectral);
+  ComplexChebyCoeff u_v(u.Ny(), u.a(), u.b(), Spectral);
+  for (int i=0; i<u.vectorDim(); ++i)
+    for (int nx=0; nx<u.numXmodes(); ++nx) {
+      if (abs(u.kx(nx)) > kxmax)
+	continue;
+      int cz = 1;
+      for (int nz=0; nz<u.numZmodes(); ++nz) {
+	if (abs(u.kz(nz)) > kzmax)
+	  continue;
+	for (int ny=0; ny<u.numYmodes(); ++ny) 
+	  u_v.set(ny, u.cmplx(nx,ny,nz,i)-v.cmplx(nx,ny,nz,i));
+	sum += cz*L2Norm2(u_v, normalize);
+	cz = 2;
+      }
+    }
+  if (!normalize)
+    sum *= u.Lx()*u.Lz();
+  return sum;
+}
+
 Complex L2InnerProduct(const FlowField& u, const BasisFunc& phi,
 		       bool normalize) {
   assert(u.xzstate()==Spectral && u.ystate()==Spectral);
@@ -1480,7 +1657,7 @@ void FlowField::nonlinearityRotational(const ChebyCoeff& Ubase,
   // f = omega x U + Omega x u + omega x u
   for (int ny=0; ny<Ny_; ++ny) { 
     //Real Uy = Ubasey[ny];
-    for (int nx=0; nx<Nx_; ++nx) 
+    for (int nx=0; nx<Nx_; ++nx) {
       for (int nz=0; nz<Nz_; ++nz) {
 	u = velocity(nx,ny,nz, 0);
 	v = velocity(nx,ny,nz, 1);
@@ -1492,8 +1669,8 @@ void FlowField::nonlinearityRotational(const ChebyCoeff& Ubase,
 	f(nx,ny,nz, 0) += omega_y*w - omega_z*v;
 	f(nx,ny,nz, 1) += omega_z*u - omega_x*w;
 	f(nx,ny,nz, 2) += omega_x*v - omega_y*u;
-
       }
+    }
   }
   //omega.makeSpectral();
   velocity.makeSpectral();
@@ -2086,7 +2263,7 @@ Real FlowField::divnorm2() const {
       div += L2Norm2(::divergence(profile(kx_,kz_)));
     }
   }
-  return sqrt(div);
+  return div;
 }
 
 /******************
@@ -2357,9 +2534,9 @@ void randomVprofile(ComplexChebyCoeff& v, Real mag, Real decay) {
     v.set(n, 0.0);
   for (n=4; n<N-2; ++n) {
     v.set(n, mag*randomComplex());
-    if (n>2)
-      mag *= decay;
+    mag *= decay;
   }
+
   for (n=Greater(N-2, 0); n<N; ++n) 
     v.set(n, 0.0);
 
@@ -2797,6 +2974,7 @@ void FlowField::saveDivProfile(int nx, int nz2, const string& filename, const Ch
   profile.save(filename);
 }
 
+*************/
 void FlowField::saveDivSpectrum(const string& filebase) const {
   string filename(filebase);
   filename += string(".asc");
@@ -2826,17 +3004,16 @@ void FlowField::saveDivSpectrum(const string& filebase) const {
       diff(v,vy);
       Real div = 0.0;
       for (int ny=0; ny<Ny; ++ny) {
-	Complex ux = cx*kx_*cmplx(nx,ny,nz,0);
-	Complex wz = cz*kz_*cmplx(nx,ny,nz,2);
-	div += norm(ux + vy[ny] + wz);
+	Complex ux = cx*Real(kx_)*cmplx(nx,ny,nz,0);
+	Complex wz = cz*Real(kz_)*cmplx(nx,ny,nz,2);
+	div += norm2(ux + vy[ny] + wz);
       }
-      os << div << ' ';
+      os << sqrt(div) << ' ';
     }
     os << endl;
   }
   os.close();
 }
-*************/
 
 /****************************************************
 // Broken for quite a while, commented out 4/21/01.
