@@ -24,7 +24,7 @@
  */
 
 #include "nsintegrator.h"
-#include "orrsommfunc.h"
+//#include "orrsommfunc.h"
 //#include <fstream> // tmp debugging need
 
 // 3rd order implicit Runge Kutta constants.
@@ -143,8 +143,10 @@ NSIntegrator::NSIntegrator(FlowField& u, const ChebyCoeff& Ubase,
   Ny_(u.numYmodes()),
   Nyd_(flags.dealias_y() ? 2*(u.numYmodes()-1)/3 + 1 : u.numYmodes()),
   Nz_(u.numZmodes()),
-  kxd_max_(flags.dealias_xz() ? 2*u.kxmax()/3 : u.kxmax()),
-  kzd_max_(flags.dealias_xz() ? 2*u.kzmax()/3 : u.kzmax()),
+  //kxd_max_(flags.dealias_xz() ? 2*(u.kxmax()/3) : u.kxmax()),
+  //kzd_max_(flags.dealias_xz() ? 2*(u.kzmax()/3) : u.kzmax()),
+  kxd_max_(flags.dealias_xz() ? u.Nx()/3-1 : u.kxmax()),
+  kzd_max_(flags.dealias_xz() ? u.Nz()/3-1 : u.kzmax()),
   Nsubsteps_(flags.numSubsteps()),
   Lx_(u.Lx()),
   Lz_(u.Lz()),
@@ -435,6 +437,50 @@ Real NSIntegrator::dPdxRef() const {return dPdxRef_;}
 Real NSIntegrator::Ubulk() const {return UbulkAct_;}
 Real NSIntegrator::UbulkRef() const {return UbulkRef_;}
 
+int NSIntegrator::kxmaxDealiased() const {return kxd_max_;}
+int NSIntegrator::kzmaxDealiased() const {return kzd_max_;}
+bool NSIntegrator::isAliasedMode(int kx, int kz) const {
+  return (abs(kx) > kxd_max_ || (abs(kz) > kzd_max_)) ? true : false;
+}
+
+
+// This function is for debugging a problme in nsintegrator::advance.
+// The problem appears when no dealiasing is done (i.e. no high-order 
+// modes are set to zero). The solution un is divergence-free immediately
+// after solving the time-stepping equations. But, transforming un to 
+// Physical and back introduces 1e-8 order changes in the kx,kz == kxmax,0 
+// and kx,kz=*,kzmax modes. These 1e-8 canges cause order-1 divergence
+// in those modes. The only way I have found to eliminate them is to
+// *always dealias* kx,kz == kxmax,* and  kx,kz=*,kzmax. 1/31/04.
+// Possibly this is a problem with FFTW or my use of it. 
+
+void NSIntegrator::debugfunc(const FlowField& u) const {
+
+  FlowField v(u);
+  v.makePhysical();
+  v.makeSpectral();
+  FlowField w(u);
+  w -= v;
+
+  cout << "L2Norm2(u)   == " << L2Norm2(u) << endl;
+  cout << "L2Norm2(v)   == " << L2Norm2(v) << endl;
+  cout << "L2Norm2(u-v) == " << L2Norm2(w) << endl;
+  cout << "L2Dist2(u,v) == " << L2Dist2(u,v) << endl;
+  cout << "    div(u)   == " << u.divergence() << endl;
+  cout << "    div(v)   == " << v.divergence() << endl;
+  cout << "    div(u-v) == " << w.divergence() << endl;
+
+  u.saveSpectrum("u");
+  v.saveSpectrum("v");
+  w.saveSpectrum("w");
+
+  u.saveDivSpectrum("ud");
+  v.saveDivSpectrum("vd");
+  w.saveDivSpectrum("wd");
+  cfpause();
+}
+  
+
 void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 
   bool verify = (flags_.verbosity == VerifyTauSolve || 
@@ -443,6 +489,9 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
   for (int stepsRemaining=nSteps; stepsRemaining>0; --stepsRemaining) {
     for (int substep=0; substep<Nsubsteps_; ++substep) {
 
+      //cout << "Before integration..." << endl;
+      //debugfunc(un);
+
       // Take a time step. Current time is t=n*dt. Calculate u,q at t=(n+1)*dt
       // and store it in un,qn. I.e. map
       // (u(n), q(n), f(n), f(n-1)) ->  (u(n+1), q(n+1), f(n+1), f(n)). 
@@ -471,6 +520,8 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	else
 	  un.nonlinearityRotational(Ubase_, Ubasey_, UbaseyT_, vortn_, fn_); 
       }
+      //cout << "After nonlinearity calc..." << endl;
+      //debugfunc(un);
 
       //cerr << "NSI: SUBSTEP==" << substep << "L2Norm(fn) == " << L2Norm(fn_) << endl;
       Real ab = alpha_[substep]/beta_[substep]; 
@@ -481,11 +532,14 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
       // Update with the time-stepping algorthim
       // Each (kx,kz) pair is an independent calculation, so parallelism
       // can be implemented at this level in the code.
+      int kxmax = un.kxmax();
+      int kzmax = un.kzmax();
       for (int nx=0; nx<Nx_; ++nx) {
 	const int kx = un.kx(nx);
 	for (int nz=0; nz<Nz_; ++nz) {
 	  const int kz = un.kz(nz);
-	  if (flags_.dealias_xz() && isAliasedMode(kx,kz)) {
+	  if ((kx==kxmax || kz==kzmax) ||
+	      (flags_.dealias_xz() && isAliasedMode(kx,kz))) {
 	    for (int ny=0; ny<Nyd_; ++ny) {
 	      un.cmplx(nx,ny,nz,0) = 0.0;
 	      un.cmplx(nx,ny,nz,1) = 0.0;
@@ -612,6 +666,8 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
 	  }
 	}
       }
+      //cout << "Check un after calcs" << endl;
+      //debugfunc(un);
     }
     t_ += dt_;
     if (flags_.verbosity == PrintTime ||
@@ -622,12 +678,20 @@ void NSIntegrator::advance(FlowField& un, FlowField& qn, int nSteps) {
     }
     alternator_ = !alternator_;  // swap Convection/Divergence if Alternating
   }
+  //cout << "After integration..." << endl;
+  //debugfunc(un);
 
   // No need to compute CFL during substeps, since variable timesteps are
   // controlled externally to NSIntegrator (by checking NSIntegrator::CFL()
   // and calling NSIntegrator::reset(dt,nu) if necessary).
   cfl_ = un.CFLfactor();
   cfl_ *= flags_.dealias_xz() ? 2.0*pi/3.0*dt_ : pi*dt_;
+  
+  //cout << "After CFL calc..." << endl;
+  //debugfunc(un);
+
+  // If using dealiasing, set flag in FlowField that compactifies IO 
+  un.dealiasIO(flags_.dealias_xz()); 
 
   if (flags_.verbosity == PrintTime ||
       flags_.verbosity == PrintAll) 
