@@ -56,13 +56,14 @@ Real divcheck(string& label, int kx, int kz, Real kxLx, Real kzLz,
   ComplexChebyCoeff tmp = diff(v);
   Real diverge=0.0;
   for (int n=N-1; n>=0; --n) 
-    diverge += norm(tmp[n] + (pi*2*(kxLx*u[n] + kzLz*w[n]))*I);
+    diverge += norm2(tmp[n] + (pi*2*(kxLx*u[n] + kzLz*w[n]))*I);
+  diverge = sqrt(diverge);
 
-  if (norm(diverge) > 1e-13 || verbose) {
+  if (diverge > 1e-13 || verbose) {
     cerr << label << endl;
     cerr << "kx, kz == " << kx << ", " << kz << endl;
     cerr << "kxLx, kzLz == " << kxLx << ", " << kzLz << endl;
-    cerr << "divergence == " << norm(diverge) << endl;
+    cerr << "divergence == " << diverge << endl;
   }
   return diverge;
 }
@@ -256,10 +257,12 @@ void TauSolver::solve_P_and_v(ChebyCoeff& P, ChebyCoeff& v, const ChebyCoeff& r,
 }
 
 
-void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v, 
+Real TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v, 
 			       const ChebyCoeff& r, const ChebyCoeff& Ry,
 			       Real sigmaNb1, Real sigmaNb, bool verbose) const {
 
+  Real error = 0.0;
+
   if (verbose) {
     cerr << "--------TauSolver::verify_P_and_v(P,v,r,Ry,sigmaNb1,sigmaNb) {" << endl;
     cerr << "nu = " << nu_ << "; lambda = " << lambda_ << endl;
@@ -271,9 +274,12 @@ void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   p_lhs *= - 4*pi*pi*(kxLx_*kxLx_ + kzLz_*kzLz_);
   p_lhs += Pyy;
 
+  Real l2err = L2Dist(p_lhs, r);
+  Real t2err = tauDist(p_lhs, r);
+  error += l2err;
   if (verbose) {
-    cerr << "Homog tauDist(P'' - k^2 P, r)   == " << tauDist(p_lhs, r) << endl;
-    cerr << "Homog  L2Dist(P'' - k^2 P, r)   == " << L2Dist(p_lhs, r) << endl;
+    cerr << "Homog tauDist(P'' - k^2 P, r)   == " << t2err << endl;
+    cerr << "Homog  L2Dist(P'' - k^2 P, r)   == " << l2err << endl;
   }    
   
   ChebyCoeff vy = diff(v);
@@ -282,14 +288,13 @@ void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   ChebyCoeff minusRy = Ry;
   minusRy *= -1.0;
 
-  Real tauError = tauDist(v_lhs, minusRy);
-  Real L2Error  = L2Dist(v_lhs, minusRy);
+  t2err = tauDist(v_lhs, minusRy);
+  l2err = L2Dist(v_lhs, minusRy);
+  error += l2err;
   if (verbose) {
-    cerr << "tauDist(v_lhs, -Ry) == " << tauError << endl;
-    cerr << " L2Dist(v_lhs, -Ry) == " << L2Error  << endl;
+    cerr << "tauDist(v_lhs, -Ry) == " << t2err << endl;
+    cerr << " L2Dist(v_lhs, -Ry) == " << l2err << endl;
   }
-  else
-    assert(L2Error < EPSILON);
 
   // Now compare P and v eqns with tau correction terms.
   ChebyCoeff sigma(N_,a_,b_,Spectral);
@@ -299,16 +304,24 @@ void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   diff(sigma, p_rhs);
   p_rhs += r;
 
+  t2err = tauDist(p_lhs, p_rhs);
+  l2err = L2Dist(p_lhs, p_rhs);
+  error += l2err;
+  if (verbose) {
+    cerr << "Homog tauDist(P'' - k^2 P, r + sigma')   == " << t2err << endl;
+    cerr << "Homog  L2Dist(P'' - k^2 P, r + sigma')   == " << l2err << endl;
+  }
+
   ChebyCoeff v_rhs(minusRy);
   v_rhs -= sigma;
+
+  t2err = tauDist(v_lhs, v_rhs);
+  l2err = L2Dist(v_lhs, v_rhs);
+  error += l2err;
   if (verbose) {
-    cerr << "Homog tauDist(P'' - k^2 P, r + sigma')   == " << tauDist(p_lhs, p_rhs) << endl;
-    cerr << "Homog  L2Dist(P'' - k^2 P, r + sigma')   == " << L2Dist(p_lhs, p_rhs) << endl;
-        
     cerr << "tauDist(v_lhs, -Ry - sigma) == " << tauDist(v_lhs, v_rhs) << endl;
     cerr << " L2Dist(v_lhs, -Ry - sigma) == " << L2Dist(v_lhs, v_rhs)  << endl;
   }
-  
 
   Real vp = v.eval_b();
   Real vm = v.eval_a();
@@ -322,15 +335,15 @@ void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   p_norm = (p_norm > EPSILON) ? p_norm : 1.0;
 
 
+  error += abs(vp)/v_norm;
+  error += abs(vm)/v_norm;
+  error += abs(vyp)/v_norm;
+  error += abs(vym)/v_norm;
+
   if (verbose) {
     cerr << " v(+/- 1) == " << vp << ' ' << vm << endl;
     cerr << "v'(+/- 1) == " << vyp << ' ' << vym << endl;
   }
-  else {
-    assert(abs(vp)/v_norm<EPSILON && abs(vm)/v_norm<EPSILON);
-    assert(abs(vyp)/v_norm<EPSILON && abs(vym)/v_norm<EPSILON);
-  }
-  
 
   //Real v_err = tauDist(v_lhs,v_rhs)/v_norm;
   //Real p_err = tauDist(p_lhs,p_rhs)/p_norm;
@@ -338,6 +351,9 @@ void TauSolver::verify_P_and_v(const ChebyCoeff& P, const ChebyCoeff& v,
   //assert(p_err < EPSILON);
   if (verbose)
     cerr << "} TauSolver::verify_P_and_v" << endl;
+
+  return error;
+	      
 }  
   
 
@@ -448,17 +464,17 @@ void TauSolver::solve(ComplexChebyCoeff& u, ComplexChebyCoeff& v,
   return;
 }
 
-void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
+Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
 		       const ComplexChebyCoeff& w, const ComplexChebyCoeff& P, 
 		       const ComplexChebyCoeff& Rx,const ComplexChebyCoeff& Ry,
 		       const ComplexChebyCoeff& Rz, bool verbose) const {
 
   Real umean = Re(u.mean());
   Real dPdx = 0.0;
-  verify(u,v,w,P,dPdx,Rx,Ry,Rz,umean,verbose);
+  return verify(u,v,w,P,dPdx,Rx,Ry,Rz,umean,verbose);
 }
   
-void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
+Real TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v, 
 		       const ComplexChebyCoeff& w, const ComplexChebyCoeff& P,
 		       Real dPdx,
 		       const ComplexChebyCoeff& Rx,const ComplexChebyCoeff& Ry,
@@ -477,7 +493,8 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   ComplexChebyCoeff lhs(N_,a_,b_,Spectral);
   ComplexChebyCoeff tmp(N_,a_,b_,Spectral);
   Real error = 0.0;
-
+  Real terr = 0.0;
+  Real lerr = 0.0;
 
   // Verify u eqn, -nu u'' + lambda u + dP/dx == Rx
   lhs = u;
@@ -490,14 +507,15 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   tmp *= I*pi2*kxLx_;
   lhs += tmp;
   lhs.re[0] += dPdx;
-  error = tauDist(lhs, Rx);
+
+  terr = tauDist(lhs, Rx);
+  lerr = L2Dist(lhs, Rx);
+  error += lerr;
   if (verbose) {
     cerr << "L2Norm(Rx) == " << L2Norm(Rx) << endl;
-    cerr << "tauDist(nu u'' - lambda u - dP/dx, -Rx) == " << error << endl;
-    cerr << " L2Dist(nu u'' - lambda u - dP/dx, -Rx) == " << L2Dist(lhs, Rx) << endl;
+    cerr << "tauDist(nu u'' - lambda u - dP/dx, -Rx) == " << terr << endl;
+    cerr << " L2Dist(nu u'' - lambda u - dP/dx, -Rx) == " << lerr << endl;
   }
-  else 
-    assert(error<EPSILON);
 
   // Verify v eqn.
   lhs = v;
@@ -507,15 +525,14 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   lhs -= tmp;
   diff(P, tmp);
   lhs += tmp;
-  error = tauDist(lhs, Ry);
+  terr = tauDist(lhs, Ry);
+  lerr = L2Dist(lhs, Ry);
+  error += lerr;
   if (verbose) {
     cerr << "L2Norm(Ry) == " << L2Norm(Ry) << endl;
-    cerr << "tauDist(nu v'' - lambda v - dP/dy, -Ry) == " << error << endl;
-    cerr << " L2Dist(nu v'' - lambda v - dP/dy, -Ry) == " << L2Dist(lhs, Ry) << endl;
+    cerr << "tauDist(nu v'' - lambda v - dP/dy, -Ry) == " << terr << endl;
+    cerr << " L2Dist(nu v'' - lambda v - dP/dy, -Ry) == " << lerr << endl;
   }
-  else
-    //assert(error<EPSILON);
-    assert(error<1.0);
 
   // Verify w eqn
   lhs = w;
@@ -526,14 +543,14 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   tmp = P;
   tmp *= I*pi2*kzLz_;
   lhs += tmp;
-  error = tauDist(lhs, Rz);
+  terr = tauDist(lhs, Rz);
+  lerr = L2Dist(lhs, Rz);
+  error += lerr;
   if (verbose) {
     cerr << "L2Norm(Rz) == " << L2Norm(Rz) << endl;
-    cerr << "tauDist(nu w'' - lambda w - dP/dz, -Rz) == " << error << endl;
-    cerr << " L2Dist(nu w'' - lambda w - dP/dz, -Rz) == " << L2Dist(lhs, Rz) << endl;
+    cerr << "tauDist(nu w'' - lambda w - dP/dz, -Rz) == " << terr << endl;
+    cerr << " L2Dist(nu w'' - lambda w - dP/dz, -Rz) == " << lerr << endl;
   }
-  else
-    assert(error<EPSILON);
 
   // Verify P eqn. P'' - kappa^2 P = div R
   diff2(P, lhs);
@@ -557,11 +574,13 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   
   r.re = r_re;
   r.im = r_im;
-  error = tauDist(lhs, r);
+  terr = tauDist(lhs, r);
+  lerr = L2Dist(lhs, r);
+  error += lerr;
   if (verbose) {
     cerr << "L2Norm(div R) == " << L2Norm(r) << endl;
-    cerr << "tauDist(P'' - k^2 P, div R)   == " << error << endl;
-    cerr << " L2Dist(P'' - k^2 P, div R)   == " << L2Dist(lhs, r) << endl;
+    cerr << "tauDist(P'' - k^2 P, div R)   == " << terr << endl;
+    cerr << " L2Dist(P'' - k^2 P, div R)   == " << lerr << endl;
   }
   // Don't assert on pressure eqn, since it's modified for tau correction.
   //else
@@ -573,53 +592,51 @@ void TauSolver::verify(const ComplexChebyCoeff& u, const ComplexChebyCoeff& v,
   for (n=0; n<N_; ++n) 
     tmp.add(n, I*pi2*(kxLx_*u[n] + kzLz_*w[n]));
 
-  error = tauNorm(tmp);
+  terr = tauNorm(tmp);
+  lerr = L2Norm(tmp);
+  error += lerr;
   if (verbose) {
-    cerr << "tauNorm(div) == " << error << endl;
-    cerr << " L2Norm(div) == " << L2Norm(tmp) << endl;
+    cerr << "tauNorm(div) == " << terr << endl;
+    cerr << " L2Norm(div) == " << lerr << endl;
   }
-  else
-    assert(error<EPSILON);
 
   // BCs
   Complex ua = u.eval_a();
   Complex ub = u.eval_b();
+  error += abs(ua) + abs(ub);
   if (verbose) 
     cerr << "u(a),u(b) == " << ua << " " << ub << endl;
-  else
-    assert(norm(ua)<EPSILON && norm(ub)<EPSILON);
+  
   
   Complex va = v.eval_a();
   Complex vb = v.eval_b();
+  error += abs(va) + abs(vb);
   if (verbose) 
     cerr << "v(a),v(b) == " << va << " " << vb << endl;
-  else
-    assert(norm(va)<EPSILON && norm(vb)<EPSILON);
 
   ComplexChebyCoeff vy = diff(v);
   Complex vya = vy.eval_a();
   Complex vyb = vy.eval_b();
+  error += abs(vya) + abs(vyb);
   if (verbose) 
     cerr << "v' at a,b == " << vya << " " << vyb << endl;
-  else
-    assert(norm(vya)<EPSILON && norm(vyb)<EPSILON);
   
   Complex wa = u.eval_a();
   Complex wb = u.eval_b();
+  error += abs(wa) + abs(wb);
   if (verbose) 
     cerr << "w(a),w(b) == " << wa << " " << wb << endl;
-  else
-    assert(norm(wa)<EPSILON && norm(wb)<EPSILON);
 
   if (verbose)
     cerr << "} TauSolver::verify(...)" << endl;
   
-  Real mean_error = norm(Re(u.mean()) - umean);
+  Real mean_error = norm2(Re(u.mean()) - umean);
   if (verbose)
-    cerr << "abs(u.mean() - umean) == " << mean_error << endl;
+    cerr << "norm2(u.mean() - umean) == " << mean_error << endl;
   else
     assert(mean_error < EPSILON);
-
+  
+  return error;
 }
   
 Real TauSolver::tauNorm(const ChebyCoeff& u) const {
