@@ -1,7 +1,7 @@
 #include <iostream>
 #include <fstream>
 #include "flowfield.h"
-#include "vector.h"
+#include "cfvector.h"
 #include "chebyshev.h"
 #include "tausolver.h"
 
